@@ -1,19 +1,21 @@
 use std::cmp::Ordering;
 use std::fs;
+use std::io::Error;
 use std::sync::Arc;
 use std::sync::Mutex;
 
 use crate::node::Node;
+use crate::progress::ORDERING;
 use crate::progress::Operation;
 use crate::progress::PAtomicInfo;
 use crate::progress::RuntimeErrors;
-use crate::progress::ORDERING;
 use crate::utils::is_filtered_out_due_to_file_time;
 use crate::utils::is_filtered_out_due_to_invert_regex;
 use crate::utils::is_filtered_out_due_to_regex;
 use rayon::iter::ParallelBridge;
 use rayon::prelude::ParallelIterator;
 use regex::Regex;
+use std::path::Path;
 use std::path::PathBuf;
 
 use std::collections::HashSet;
@@ -21,10 +23,11 @@ use std::collections::HashSet;
 use crate::node::build_node;
 use std::fs::DirEntry;
 
+use crate::node::FileTime;
 use crate::platform::get_metadata;
 
 #[derive(Debug)]
-pub enum Operater {
+pub enum Operator {
     Equal = 0,
     LessThan = 1,
     GreaterThan = 2,
@@ -35,11 +38,12 @@ pub struct WalkData<'a> {
     pub filter_regex: &'a [Regex],
     pub invert_filter_regex: &'a [Regex],
     pub allowed_filesystems: HashSet<u64>,
-    pub filter_modified_time: (Operater, i64),
-    pub filter_accessed_time: (Operater, i64),
-    pub filter_changed_time: (Operater, i64),
+    pub filter_modified_time: Option<(Operator, i64)>,
+    pub filter_accessed_time: Option<(Operator, i64)>,
+    pub filter_changed_time: Option<(Operator, i64)>,
     pub use_apparent_size: bool,
     pub by_filecount: bool,
+    pub by_filetime: &'a Option<FileTime>,
     pub ignore_hidden: bool,
     pub follow_links: bool,
     pub progress_data: Arc<PAtomicInfo>,
@@ -57,19 +61,15 @@ pub fn walk_it(dirs: HashSet<PathBuf>, walk_data: &WalkData) -> Vec<Node> {
 
             prog_data.state.store(Operation::PREPARING, ORDERING);
 
-            clean_inodes(node, &mut inodes, walk_data.use_apparent_size)
+            clean_inodes(node, &mut inodes, walk_data)
         })
         .collect();
     top_level_nodes
 }
 
 // Remove files which have the same inode, we don't want to double count them.
-fn clean_inodes(
-    x: Node,
-    inodes: &mut HashSet<(u64, u64)>,
-    use_apparent_size: bool,
-) -> Option<Node> {
-    if !use_apparent_size {
+fn clean_inodes(x: Node, inodes: &mut HashSet<(u64, u64)>, walk_data: &WalkData) -> Option<Node> {
+    if !walk_data.use_apparent_size {
         if let Some(id) = x.inode_device {
             if !inodes.insert(id) {
                 return None;
@@ -82,12 +82,25 @@ fn clean_inodes(
     tmp.sort_by(sort_by_inode);
     let new_children: Vec<_> = tmp
         .into_iter()
-        .filter_map(|c| clean_inodes(c, inodes, use_apparent_size))
+        .filter_map(|c| clean_inodes(c, inodes, walk_data))
         .collect();
 
+    let actual_size = if walk_data.by_filetime.is_some() {
+        // If by_filetime is Some, directory 'size' is the maximum filetime among child files instead of disk size
+        new_children
+            .iter()
+            .map(|c| c.size)
+            .chain(std::iter::once(x.size))
+            .max()
+            .unwrap_or(0)
+    } else {
+        // If by_filetime is None, directory 'size' is the sum of disk sizes or file counts of child files
+        x.size + new_children.iter().map(|c| c.size).sum::<u64>()
+    };
+
     Some(Node {
         name: x.name,
-        size: x.size + new_children.iter().map(|c| c.size).sum::<u64>(),
+        size: actual_size,
         children: new_children,
         inode_device: x.inode_device,
         depth: x.depth,
@@ -112,31 +125,62 @@ fn sort_by_inode(a: &Node, b: &Node) -> std::cmp::Ordering {
     }
 }
 
-fn ignore_file(entry: &DirEntry, walk_data: &WalkData) -> bool {
-    let is_dot_file = entry.file_name().to_str().unwrap_or("").starts_with('.');
-    let is_ignored_path = walk_data.ignore_directories.contains(&entry.path());
+// Check if `path` is inside ignored directory
+fn is_ignored_path(path: &Path, walk_data: &WalkData) -> bool {
+    if walk_data.ignore_directories.contains(path) {
+        return true;
+    }
 
-    let size_inode_device = get_metadata(entry.path(), false);
-    if let Some((_size, Some((_id, dev)), (modified_time, accessed_time, changed_time))) =
-        size_inode_device
-    {
-        if !walk_data.allowed_filesystems.is_empty()
-            && !walk_data.allowed_filesystems.contains(&dev)
-        {
+    // Entry is inside an ignored absolute path
+    // Absolute paths should be canonicalized before being added to `WalkData.ignore_directories`
+    for ignored_path in walk_data.ignore_directories.iter() {
+        if !ignored_path.is_absolute() {
+            continue;
+        }
+        let absolute_entry_path = std::fs::canonicalize(path).unwrap_or_default();
+        if absolute_entry_path.starts_with(ignored_path) {
             return true;
         }
-        if entry.path().is_file()
-            && [
-                (&walk_data.filter_modified_time, modified_time),
-                (&walk_data.filter_accessed_time, accessed_time),
-                (&walk_data.filter_changed_time, changed_time),
-            ]
-            .iter()
-            .any(|(filter_time, actual_time)| {
-                is_filtered_out_due_to_file_time(filter_time, *actual_time)
-            })
-        {
-            return true;
+    }
+
+    false
+}
+
+fn ignore_file(entry: &DirEntry, walk_data: &WalkData) -> bool {
+    if is_ignored_path(&entry.path(), walk_data) {
+        return true;
+    }
+
+    let is_dot_file = entry.file_name().to_str().unwrap_or("").starts_with('.');
+    let follow_links = walk_data.follow_links && entry.file_type().is_ok_and(|ft| ft.is_symlink());
+
+    if !walk_data.allowed_filesystems.is_empty() {
+        let size_inode_device = get_metadata(entry.path(), false, follow_links);
+        if let Some((_size, Some((_id, dev)), _gunk)) = size_inode_device {
+            if !walk_data.allowed_filesystems.contains(&dev) {
+                return true;
+            }
+        }
+    }
+    if walk_data.filter_accessed_time.is_some()
+        || walk_data.filter_modified_time.is_some()
+        || walk_data.filter_changed_time.is_some()
+    {
+        let size_inode_device = get_metadata(entry.path(), false, follow_links);
+        if let Some((_, _, (modified_time, accessed_time, changed_time))) = size_inode_device {
+            if entry.path().is_file()
+                && [
+                    (&walk_data.filter_modified_time, modified_time),
+                    (&walk_data.filter_accessed_time, accessed_time),
+                    (&walk_data.filter_changed_time, changed_time),
+                ]
+                .iter()
+                .any(|(filter_time, actual_time)| {
+                    is_filtered_out_due_to_file_time(filter_time, *actual_time)
+                })
+            {
+                return true;
+            }
         }
     }
 
@@ -155,17 +199,13 @@ fn ignore_file(entry: &DirEntry, walk_data: &WalkData) -> bool {
         return true;
     }
 
-    (is_dot_file && walk_data.ignore_hidden) || is_ignored_path
+    is_dot_file && walk_data.ignore_hidden
 }
 
 fn walk(dir: PathBuf, walk_data: &WalkData, depth: usize) -> Option<Node> {
     let prog_data = &walk_data.progress_data;
     let errors = &walk_data.errors;
 
-    if errors.lock().unwrap().abort {
-        return None;
-    }
-
     let children = if dir.is_dir() {
         let read_dir = fs::read_dir(&dir);
         match read_dir {
@@ -211,8 +251,9 @@ fn walk(dir: PathBuf, walk_data: &WalkData, depth: usize) -> Option<Node> {
                                 }
                             }
                             Err(ref failed) => {
-                                let mut editable_error = errors.lock().unwrap();
-                                editable_error.no_permissions.insert(failed.to_string());
+                                if handle_error_and_retry(failed, &dir, walk_data) {
+                                    return walk(dir.clone(), walk_data, depth);
+                                }
                             }
                         }
                         None
@@ -220,21 +261,11 @@ fn walk(dir: PathBuf, walk_data: &WalkData, depth: usize) -> Option<Node> {
                     .collect()
             }
             Err(failed) => {
-                let mut editable_error = errors.lock().unwrap();
-                match failed.kind() {
-                    std::io::ErrorKind::PermissionDenied => {
-                        editable_error
-                            .no_permissions
-                            .insert(dir.to_string_lossy().into());
-                    }
-                    std::io::ErrorKind::NotFound => {
-                        editable_error.file_not_found.insert(failed.to_string());
-                    }
-                    _ => {
-                        editable_error.unknown_error.insert(failed.to_string());
-                    }
+                if handle_error_and_retry(&failed, &dir, walk_data) {
+                    return walk(dir, walk_data, depth);
+                } else {
+                    vec![]
                 }
-                vec![]
             }
         }
     } else {
@@ -245,7 +276,46 @@ fn walk(dir: PathBuf, walk_data: &WalkData, depth: usize) -> Option<Node> {
         }
         vec![]
     };
-    build_node(dir, children, false, false, depth, walk_data)
+    let is_symlink = if walk_data.follow_links {
+        match fs::symlink_metadata(&dir) {
+            Ok(metadata) => metadata.file_type().is_symlink(),
+            Err(_) => false,
+        }
+    } else {
+        false
+    };
+    build_node(dir, children, is_symlink, false, depth, walk_data)
+}
+
+fn handle_error_and_retry(failed: &Error, dir: &Path, walk_data: &WalkData) -> bool {
+    let mut editable_error = walk_data.errors.lock().unwrap();
+    match failed.kind() {
+        std::io::ErrorKind::PermissionDenied => {
+            editable_error
+                .no_permissions
+                .insert(dir.to_string_lossy().into());
+        }
+        std::io::ErrorKind::InvalidInput => {
+            editable_error
+                .no_permissions
+                .insert(dir.to_string_lossy().into());
+        }
+        std::io::ErrorKind::NotFound => {
+            editable_error.file_not_found.insert(failed.to_string());
+        }
+        std::io::ErrorKind::Interrupted => {
+            editable_error.interrupted_error += 1;
+            if editable_error.interrupted_error > 3 {
+                panic!("Multiple Interrupted Errors occurred while scanning filesystem. Aborting");
+            } else {
+                return true;
+            }
+        }
+        _ => {
+            editable_error.unknown_error.insert(failed.to_string());
+        }
+    }
+    false
 }
 
 mod tests {
@@ -264,17 +334,43 @@ mod tests {
         }
     }
 
+    #[cfg(test)]
+    fn create_walker<'a>(use_apparent_size: bool) -> WalkData<'a> {
+        use crate::PIndicator;
+        let indicator = PIndicator::build_me();
+        WalkData {
+            ignore_directories: HashSet::new(),
+            filter_regex: &[],
+            invert_filter_regex: &[],
+            allowed_filesystems: HashSet::new(),
+            filter_modified_time: Some((Operator::GreaterThan, 0)),
+            filter_accessed_time: Some((Operator::GreaterThan, 0)),
+            filter_changed_time: Some((Operator::GreaterThan, 0)),
+            use_apparent_size,
+            by_filecount: false,
+            by_filetime: &None,
+            ignore_hidden: false,
+            follow_links: false,
+            progress_data: indicator.data.clone(),
+            errors: Arc::new(Mutex::new(RuntimeErrors::default())),
+        }
+    }
+
     #[test]
     #[allow(clippy::redundant_clone)]
     fn test_should_ignore_file() {
         let mut inodes = HashSet::new();
         let n = create_node();
+        let walkdata = create_walker(false);
 
         // First time we insert the node
-        assert_eq!(clean_inodes(n.clone(), &mut inodes, false), Some(n.clone()));
+        assert_eq!(
+            clean_inodes(n.clone(), &mut inodes, &walkdata),
+            Some(n.clone())
+        );
 
         // Second time is a duplicate - we ignore it
-        assert_eq!(clean_inodes(n.clone(), &mut inodes, false), None);
+        assert_eq!(clean_inodes(n.clone(), &mut inodes, &walkdata), None);
     }
 
     #[test]
@@ -282,10 +378,17 @@ mod tests {
     fn test_should_not_ignore_files_if_using_apparent_size() {
         let mut inodes = HashSet::new();
         let n = create_node();
+        let walkdata = create_walker(true);
 
         // If using apparent size we include Nodes, even if duplicate inodes
-        assert_eq!(clean_inodes(n.clone(), &mut inodes, true), Some(n.clone()));
-        assert_eq!(clean_inodes(n.clone(), &mut inodes, true), Some(n.clone()));
+        assert_eq!(
+            clean_inodes(n.clone(), &mut inodes, &walkdata),
+            Some(n.clone())
+        );
+        assert_eq!(
+            clean_inodes(n.clone(), &mut inodes, &walkdata),
+            Some(n.clone())
+        );
     }
 
     #[test]
