@@ -1,4 +1,5 @@
 use crate::display_node::DisplayNode;
+use crate::node::FileTime;
 
 use ansi_term::Colour::Red;
 use lscolors::{LsColors, Style};
@@ -7,23 +8,27 @@ use unicode_width::UnicodeWidthStr;
 
 use stfu8::encode_u8;
 
+use chrono::{DateTime, Local, TimeZone, Utc};
 use std::cmp::max;
 use std::cmp::min;
 use std::fs;
-use std::iter::repeat;
+use std::iter::repeat_n;
 use std::path::Path;
 use thousands::Separable;
 
-pub static UNITS: [char; 4] = ['T', 'G', 'M', 'K'];
+pub static UNITS: [char; 5] = ['P', 'T', 'G', 'M', 'K'];
 static BLOCKS: [char; 5] = ['█', '▓', '▒', '░', ' '];
+const FILETIME_SHOW_LENGTH: usize = 19;
 
 pub struct InitialDisplayData {
     pub short_paths: bool,
     pub is_reversed: bool,
     pub colors_on: bool,
     pub by_filecount: bool,
+    pub by_filetime: Option<FileTime>,
     pub is_screen_reader: bool,
-    pub iso: bool,
+    pub output_format: String,
+    pub bars_on_right: bool,
 }
 
 pub struct DisplayData {
@@ -66,11 +71,7 @@ impl DisplayData {
 
     fn percent_size(&self, node: &DisplayNode) -> f32 {
         let result = node.size as f32 / self.base_size as f32;
-        if result.is_normal() {
-            result
-        } else {
-            0.0
-        }
+        if result.is_normal() { result } else { 0.0 }
     }
 }
 
@@ -98,7 +99,13 @@ impl DrawData<'_> {
         let mut new_bar = "".to_string();
         let idx = 5 - level.clamp(1, 4);
 
-        for c in self.percent_bar.chars() {
+        let itr: Box<dyn Iterator<Item = char>> = if self.display_data.initial.bars_on_right {
+            Box::new(self.percent_bar.chars())
+        } else {
+            Box::new(self.percent_bar.chars().rev())
+        };
+
+        for c in itr {
             num_not_my_bar -= 1;
             if num_not_my_bar <= 0 {
                 new_bar.push(BLOCKS[0]);
@@ -108,7 +115,11 @@ impl DrawData<'_> {
                 new_bar.push(c);
             }
         }
-        new_bar
+        if self.display_data.initial.bars_on_right {
+            new_bar
+        } else {
+            new_bar.chars().rev().collect()
+        }
     }
 }
 
@@ -119,19 +130,13 @@ pub fn draw_it(
     root_node: &DisplayNode,
     skip_total: bool,
 ) {
-    let biggest = match skip_total {
-        false => root_node,
-        true => root_node
-            .get_children_from_node(false)
-            .next()
-            .unwrap_or(root_node),
-    };
-
     let num_chars_needed_on_left_most = if idd.by_filecount {
-        let max_size = biggest.size;
+        let max_size = root_node.size;
         max_size.separate_with_commas().chars().count()
+    } else if idd.by_filetime.is_some() {
+        FILETIME_SHOW_LENGTH
     } else {
-        find_biggest_size_str(root_node, idd.iso)
+        find_biggest_size_str(root_node, &idd.output_format)
     };
 
     assert!(
@@ -150,12 +155,12 @@ pub fn draw_it(
         allowed_width - longest_string_length - 7
     };
 
-    let first_size_bar = repeat(BLOCKS[0]).take(max_bar_length).collect();
+    let first_size_bar = repeat_n(BLOCKS[0], max_bar_length).collect();
 
     let display_data = DisplayData {
         initial: idd,
         num_chars_needed_on_left_most,
-        base_size: biggest.size,
+        base_size: root_node.size,
         longest_string_length,
         ls_colors: LsColors::from_env().unwrap_or_default(),
     };
@@ -179,10 +184,12 @@ pub fn draw_it(
     }
 }
 
-fn find_biggest_size_str(node: &DisplayNode, iso: bool) -> usize {
-    let mut mx = human_readable_number(node.size, iso).chars().count();
+fn find_biggest_size_str(node: &DisplayNode, output_format: &str) -> usize {
+    let mut mx = human_readable_number(node.size, output_format)
+        .chars()
+        .count();
     for n in node.children.iter() {
-        mx = max(mx, find_biggest_size_str(n, iso));
+        mx = max(mx, find_biggest_size_str(n, output_format));
     }
     mx
 }
@@ -262,7 +269,7 @@ fn clean_indentation_string(s: &str) -> String {
     is
 }
 
-fn get_printable_name<P: AsRef<Path>>(dir_name: &P, short_paths: bool) -> String {
+pub fn get_printable_name<P: AsRef<Path>>(dir_name: &P, short_paths: bool) -> String {
     let dir_name = dir_name.as_ref();
     let printable_name = {
         if short_paths {
@@ -291,12 +298,9 @@ fn pad_or_trim_filename(node: &DisplayNode, indent: &str, display_data: &Display
     );
 
     // Add spaces after the filename so we can draw the % used bar chart.
-    let name_and_padding = name
-        + " "
-            .repeat(display_data.longest_string_length - width)
-            .as_str();
-
-    name_and_padding
+    name + " "
+        .repeat(display_data.longest_string_length - width)
+        .as_str()
 }
 
 fn maybe_trim_filename(name_in: String, indent: &str, display_data: &DisplayData) -> String {
@@ -329,6 +333,8 @@ pub fn format_string(
     if display_data.initial.is_screen_reader {
         // if screen_reader then bars is 'depth'
         format!("{pretty_name} {bars} {pretty_size}{percent}")
+    } else if display_data.initial.by_filetime.is_some() {
+        format!("{pretty_size} {indent}{pretty_name}")
     } else {
         format!("{pretty_size} {indent} {pretty_name}{percent}")
     }
@@ -363,8 +369,10 @@ fn get_name_percent(
 fn get_pretty_size(node: &DisplayNode, is_biggest: bool, display_data: &DisplayData) -> String {
     let output = if display_data.initial.by_filecount {
         node.size.separate_with_commas()
+    } else if display_data.initial.by_filetime.is_some() {
+        get_pretty_file_modified_time(node.size as i64)
     } else {
-        human_readable_number(node.size, display_data.initial.iso)
+        human_readable_number(node.size, &display_data.initial.output_format)
     };
     let spaces_to_add = display_data.num_chars_needed_on_left_most - output.chars().count();
     let output = " ".repeat(spaces_to_add) + output.as_str();
@@ -376,6 +384,14 @@ fn get_pretty_size(node: &DisplayNode, is_biggest: bool, display_data: &DisplayD
     }
 }
 
+fn get_pretty_file_modified_time(timestamp: i64) -> String {
+    let datetime: DateTime<Utc> = Utc.timestamp_opt(timestamp, 0).unwrap();
+
+    let local_datetime = datetime.with_timezone(&Local);
+
+    local_datetime.format("%Y-%m-%dT%H:%M:%S").to_string()
+}
+
 fn get_pretty_name(
     node: &DisplayNode,
     name_and_padding: String,
@@ -396,19 +412,51 @@ fn get_pretty_name(
     }
 }
 
-pub fn human_readable_number(size: u64, iso: bool) -> String {
+// If we are working with SI units or not
+pub fn get_type_of_thousand(output_str: &str) -> u64 {
+    if output_str.is_empty() {
+        1024
+    } else if output_str == "si" {
+        1000
+    } else if output_str.contains('i') || output_str.len() == 1 {
+        1024
+    } else {
+        1000
+    }
+}
+
+pub fn get_number_format(output_str: &str) -> Option<(u64, char)> {
+    if output_str.starts_with('b') {
+        return Some((1, 'B'));
+    }
     for (i, u) in UNITS.iter().enumerate() {
-        let num: u64 = if iso { 1000 } else { 1024 };
-        let marker = num.pow((UNITS.len() - i) as u32);
-        if size >= marker {
-            if size / marker < 10 {
-                return format!("{:.1}{}", (size as f32 / marker as f32), u);
-            } else {
-                return format!("{}{}", (size / marker), u);
+        if output_str.starts_with((*u).to_ascii_lowercase()) {
+            let marker = get_type_of_thousand(output_str).pow((UNITS.len() - i) as u32);
+            return Some((marker, *u));
+        }
+    }
+    None
+}
+
+pub fn human_readable_number(size: u64, output_str: &str) -> String {
+    match get_number_format(output_str) {
+        Some((x, u)) => {
+            format!("{}{}", (size / x), u)
+        }
+        None => {
+            for (i, u) in UNITS.iter().enumerate() {
+                let marker = get_type_of_thousand(output_str).pow((UNITS.len() - i) as u32);
+                if size >= marker {
+                    if size / marker < 10 {
+                        return format!("{:.1}{}", (size as f32 / marker as f32), u);
+                    } else {
+                        return format!("{}{}", (size / marker), u);
+                    }
+                }
             }
+            format!("{size}B")
         }
     }
-    format!("{size}B")
 }
 
 mod tests {
@@ -424,8 +472,10 @@ mod tests {
             is_reversed: false,
             colors_on: false,
             by_filecount: false,
+            by_filetime: None,
             is_screen_reader: false,
-            iso: false,
+            output_format: "".into(),
+            bars_on_right: false,
         };
         DisplayData {
             initial,
@@ -491,21 +541,133 @@ mod tests {
 
     #[test]
     fn test_human_readable_number() {
-        assert_eq!(human_readable_number(1, false), "1B");
-        assert_eq!(human_readable_number(956, false), "956B");
-        assert_eq!(human_readable_number(1004, false), "1004B");
-        assert_eq!(human_readable_number(1024, false), "1.0K");
-        assert_eq!(human_readable_number(1536, false), "1.5K");
-        assert_eq!(human_readable_number(1024 * 512, false), "512K");
-        assert_eq!(human_readable_number(1024 * 1024, false), "1.0M");
+        assert_eq!(human_readable_number(1, ""), "1B");
+        assert_eq!(human_readable_number(956, ""), "956B");
+        assert_eq!(human_readable_number(1004, ""), "1004B");
+        assert_eq!(human_readable_number(1024, ""), "1.0K");
+        assert_eq!(human_readable_number(1536, ""), "1.5K");
+        assert_eq!(human_readable_number(1024 * 512, ""), "512K");
+        assert_eq!(human_readable_number(1024 * 1024, ""), "1.0M");
+        assert_eq!(human_readable_number(1024 * 1024 * 1024 - 1, ""), "1023M");
+        assert_eq!(human_readable_number(1024 * 1024 * 1024 * 20, ""), "20G");
+        assert_eq!(human_readable_number(1024 * 1024 * 1024 * 1024, ""), "1.0T");
         assert_eq!(
-            human_readable_number(1024 * 1024 * 1024 - 1, false),
-            "1023M"
+            human_readable_number(1024 * 1024 * 1024 * 1024 * 234, ""),
+            "234T"
         );
-        assert_eq!(human_readable_number(1024 * 1024 * 1024 * 20, false), "20G");
         assert_eq!(
-            human_readable_number(1024 * 1024 * 1024 * 1024, false),
-            "1.0T"
+            human_readable_number(1024 * 1024 * 1024 * 1024 * 1024, ""),
+            "1.0P"
         );
     }
+
+    #[test]
+    fn test_human_readable_number_si() {
+        assert_eq!(human_readable_number(1024 * 100, ""), "100K");
+        assert_eq!(human_readable_number(1024 * 100, "si"), "102K");
+    }
+
+    // Refer to https://en.wikipedia.org/wiki/Byte#Multiple-byte_units
+    #[test]
+    fn test_human_readable_number_kb() {
+        let hrn = human_readable_number;
+        assert_eq!(hrn(1023, "b"), "1023B");
+        assert_eq!(hrn(1000 * 1000, "bytes"), "1000000B");
+        assert_eq!(hrn(1023, "kb"), "1K");
+        assert_eq!(hrn(1023, "k"), "0K");
+        assert_eq!(hrn(1023, "kib"), "0K");
+        assert_eq!(hrn(1024, "kib"), "1K");
+        assert_eq!(hrn(1024 * 512, "kib"), "512K");
+        assert_eq!(hrn(1024 * 1024, "kib"), "1024K");
+        assert_eq!(hrn(1024 * 1000 * 1000 * 20, "kib"), "20000000K");
+        assert_eq!(hrn(1024 * 1024 * 1000 * 20, "mib"), "20000M");
+        assert_eq!(hrn(1024 * 1024 * 1024 * 20, "gib"), "20G");
+    }
+
+    #[cfg(test)]
+    fn build_draw_data(disp: &DisplayData, size: u32) -> (DrawData<'_>, DisplayNode) {
+        let n = DisplayNode {
+            name: PathBuf::from("/short"),
+            size: 2_u64.pow(size),
+            children: vec![],
+        };
+        let first_size_bar = repeat_n(BLOCKS[0], 13).collect();
+        let dd = DrawData {
+            indent: "".into(),
+            percent_bar: first_size_bar,
+            display_data: disp,
+        };
+        (dd, n)
+    }
+
+    #[test]
+    fn test_draw_data() {
+        let disp = &get_fake_display_data(20);
+        let (dd, n) = build_draw_data(disp, 12);
+        let bar = dd.generate_bar(&n, 1);
+        assert_eq!(bar, "█████████████");
+    }
+
+    #[test]
+    fn test_draw_data2() {
+        let disp = &get_fake_display_data(20);
+        let (dd, n) = build_draw_data(disp, 11);
+        let bar = dd.generate_bar(&n, 2);
+        assert_eq!(bar, "███████░░░░░░");
+    }
+    #[test]
+    fn test_draw_data3() {
+        let mut disp = get_fake_display_data(20);
+        let (dd, n) = build_draw_data(&disp, 11);
+        let bar = dd.generate_bar(&n, 3);
+        assert_eq!(bar, "███████▒▒▒▒▒▒");
+
+        disp.initial.bars_on_right = true;
+        let (dd, n) = build_draw_data(&disp, 11);
+        let bar = dd.generate_bar(&n, 3);
+        assert_eq!(bar, "▒▒▒▒▒▒███████")
+    }
+    #[test]
+    fn test_draw_data4() {
+        let disp = &get_fake_display_data(20);
+        let (dd, n) = build_draw_data(disp, 10);
+        // After 4 we have no more levels of shading so 4+ is the same
+        let bar = dd.generate_bar(&n, 4);
+        assert_eq!(bar, "████▓▓▓▓▓▓▓▓▓");
+        let bar = dd.generate_bar(&n, 5);
+        assert_eq!(bar, "████▓▓▓▓▓▓▓▓▓");
+    }
+
+    #[test]
+    fn test_get_pretty_file_modified_time() {
+        // Create a timestamp for 2023-07-12 00:00:00 in local time
+        let local_dt = Local.with_ymd_and_hms(2023, 7, 12, 0, 0, 0).unwrap();
+        let timestamp = local_dt.timestamp();
+
+        // Format expected output
+        let expected_output = local_dt.format("%Y-%m-%dT%H:%M:%S").to_string();
+
+        assert_eq!(get_pretty_file_modified_time(timestamp), expected_output);
+
+        // Test another timestamp
+        let local_dt = Local.with_ymd_and_hms(2020, 1, 1, 12, 0, 0).unwrap();
+        let timestamp = local_dt.timestamp();
+        let expected_output = local_dt.format("%Y-%m-%dT%H:%M:%S").to_string();
+
+        assert_eq!(get_pretty_file_modified_time(timestamp), expected_output);
+
+        // Test timestamp for epoch start (1970-01-01T00:00:00)
+        let local_dt = Local.with_ymd_and_hms(1970, 1, 1, 0, 0, 0).unwrap();
+        let timestamp = local_dt.timestamp();
+        let expected_output = local_dt.format("%Y-%m-%dT%H:%M:%S").to_string();
+
+        assert_eq!(get_pretty_file_modified_time(timestamp), expected_output);
+
+        // Test a future timestamp
+        let local_dt = Local.with_ymd_and_hms(2030, 12, 25, 6, 30, 0).unwrap();
+        let timestamp = local_dt.timestamp();
+        let expected_output = local_dt.format("%Y-%m-%dT%H:%M:%S").to_string();
+
+        assert_eq!(get_pretty_file_modified_time(timestamp), expected_output);
+    }
 }
